@@ -5,6 +5,7 @@ FITS files, individual HDUs, FITS headers, or just FITS data.
 
 Used to implement the fitsdiff program.
 """
+
 import fnmatch
 import glob
 import io
@@ -19,12 +20,7 @@ from itertools import islice
 import numpy as np
 
 from astropy import __version__
-from astropy.utils.diff import (
-    diff_values,
-    fixed_width_indent,
-    report_diff_values,
-    where_not_allclose,
-)
+from astropy.utils.diff import diff_values, report_diff_values, where_not_allclose
 from astropy.utils.misc import NOT_OVERWRITING_MSG
 
 from .card import BLANK_CARD, Card
@@ -77,7 +73,6 @@ class _BaseDiff:
         appropriate subclass of ``_BaseDiff`` for the objects being compared
         (for example, use `HeaderDiff` to compare two `Header` objects.
         """
-
         self.a = a
         self.b = b
 
@@ -92,7 +87,6 @@ class _BaseDiff:
         A ``_BaseDiff`` object acts as `True` in a boolean context if the two
         objects compared are different.  Otherwise it acts as `False`.
         """
-
         return not self.identical
 
     @classmethod
@@ -112,7 +106,6 @@ class _BaseDiff:
             >>> list(hd.ignore_keywords)
             ['*']
         """
-
         sig = signature(cls.__init__)
         # The first 3 arguments of any Diff initializer are self, a, and b.
         kwargs = {}
@@ -132,7 +125,6 @@ class _BaseDiff:
         attribute, which contains a non-empty value if and only if some
         difference was found between the two objects being compared.
         """
-
         return not any(
             getattr(self, attr) for attr in self.__dict__ if attr.startswith("diff_")
         )
@@ -163,7 +155,6 @@ class _BaseDiff:
         -------
         report : str or None
         """
-
         return_string = False
         filepath = None
 
@@ -191,7 +182,7 @@ class _BaseDiff:
             return fileobj.getvalue()
 
     def _writeln(self, text):
-        self._fileobj.write(fixed_width_indent(text, self._indent) + "\n")
+        self._fileobj.write(textwrap.indent(text, self._indent * "  ") + "\n")
 
     def _diff(self):
         raise NotImplementedError
@@ -291,16 +282,11 @@ class FITSDiff(_BaseDiff):
             Ignore all cards that are blank, i.e. they only contain
             whitespace (default: True).
         """
-
         if isinstance(a, (str, os.PathLike)):
             try:
                 a = fitsopen(a)
             except Exception as exc:
-                raise OSError(
-                    "error opening file a ({}): {}: {}".format(
-                        a, exc.__class__.__name__, exc.args[0]
-                    )
-                )
+                raise OSError(f"error opening file a ({a})") from exc
             close_a = True
         else:
             close_a = False
@@ -309,11 +295,7 @@ class FITSDiff(_BaseDiff):
             try:
                 b = fitsopen(b)
             except Exception as exc:
-                raise OSError(
-                    "error opening file b ({}): {}: {}".format(
-                        b, exc.__class__.__name__, exc.args[0]
-                    )
-                )
+                raise OSError(f"error opening file b ({b})") from exc
             close_b = True
         else:
             close_b = False
@@ -369,20 +351,10 @@ class FITSDiff(_BaseDiff):
             a_names = [hdu.name for hdu in self.a]
             b_names = [hdu.name for hdu in self.b]
             for pattern in self.ignore_hdu_patterns:
-                self.a = HDUList(
-                    [
-                        h
-                        for h in self.a
-                        if h.name not in fnmatch.filter(a_names, pattern)
-                    ]
-                )
-                self.b = HDUList(
-                    [
-                        h
-                        for h in self.b
-                        if h.name not in fnmatch.filter(b_names, pattern)
-                    ]
-                )
+                a_ignored = fnmatch.filter(a_names, pattern)
+                self.a = HDUList([h for h in self.a if h.name not in a_ignored])
+                b_ignored = fnmatch.filter(b_names, pattern)
+                self.b = HDUList([h for h in self.b if h.name not in b_ignored])
 
         # For now, just compare the extensions one by one in order.
         # Might allow some more sophisticated types of diffing later.
@@ -412,49 +384,38 @@ class FITSDiff(_BaseDiff):
 
         if self.ignore_hdus:
             ignore_hdus = " ".join(sorted(self.ignore_hdus))
-            self._writeln(f" HDU(s) not to be compared:\n{wrapper.fill(ignore_hdus)}")
+            self._writeln(" HDU(s) not to be compared:\n" + wrapper.fill(ignore_hdus))
 
         if self.ignore_hdu_patterns:
             ignore_hdu_patterns = " ".join(sorted(self.ignore_hdu_patterns))
             self._writeln(
-                " HDU(s) not to be compared:\n{}".format(
-                    wrapper.fill(ignore_hdu_patterns)
-                )
+                " HDU(s) not to be compared:\n" + wrapper.fill(ignore_hdu_patterns)
             )
 
         if self.ignore_keywords:
             ignore_keywords = " ".join(sorted(self.ignore_keywords))
             self._writeln(
-                " Keyword(s) not to be compared:\n{}".format(
-                    wrapper.fill(ignore_keywords)
-                )
+                " Keyword(s) not to be compared:\n" + wrapper.fill(ignore_keywords)
             )
 
         if self.ignore_comments:
             ignore_comments = " ".join(sorted(self.ignore_comments))
             self._writeln(
-                " Keyword(s) whose comments are not to be compared:\n{}".format(
-                    wrapper.fill(ignore_comments)
-                )
+                " Keyword(s) whose comments are not to be compared:\n"
+                + wrapper.fill(ignore_comments)
             )
 
         if self.ignore_fields:
             ignore_fields = " ".join(sorted(self.ignore_fields))
             self._writeln(
-                " Table column(s) not to be compared:\n{}".format(
-                    wrapper.fill(ignore_fields)
-                )
+                " Table column(s) not to be compared:\n" + wrapper.fill(ignore_fields)
             )
 
         self._writeln(
-            " Maximum number of different data values to be reported: {}".format(
-                self.numdiffs
-            )
+            f" Maximum number of different data values to be reported: {self.numdiffs}"
         )
         self._writeln(
-            " Relative tolerance: {}, Absolute tolerance: {}".format(
-                self.rtol, self.atol
-            )
+            f" Relative tolerance: {self.rtol}, Absolute tolerance: {self.atol}"
         )
 
         if self.diff_hdu_count:
@@ -584,7 +545,6 @@ class HDUDiff(_BaseDiff):
             Ignore all cards that are blank, i.e. they only contain
             whitespace (default: True).
         """
-
         self.ignore_keywords = {k.upper() for k in ignore_keywords}
         self.ignore_comments = {k.upper() for k in ignore_comments}
         self.ignore_fields = {k.upper() for k in ignore_fields}
@@ -795,7 +755,6 @@ class HeaderDiff(_BaseDiff):
             Ignore all cards that are blank, i.e. they only contain
             whitespace (default: True).
         """
-
         self.ignore_keywords = {k.upper() for k in ignore_keywords}
         self.ignore_comments = {k.upper() for k in ignore_comments}
 
@@ -1081,7 +1040,6 @@ class ImageDataDiff(_BaseDiff):
 
             .. versionadded:: 2.0
         """
-
         self.numdiffs = numdiffs
         self.rtol = rtol
         self.atol = atol
@@ -1168,9 +1126,8 @@ class ImageDataDiff(_BaseDiff):
         if self.diff_total > self.numdiffs:
             self._writeln(" ...")
         self._writeln(
-            " {} different pixels found ({:.2%} different).".format(
-                self.diff_total, self.diff_ratio
-            )
+            f" {self.diff_total} different pixels found "
+            f"({self.diff_ratio:.2%} different)."
         )
 
 
@@ -1213,7 +1170,6 @@ class RawDataDiff(ImageDataDiff):
             are kept in memory or output.  If a negative value is given, then
             numdiffs is treated as unlimited (default: 10).
         """
-
         self.diff_dimensions = ()
         self.diff_bytes = []
 
@@ -1257,9 +1213,8 @@ class RawDataDiff(ImageDataDiff):
 
         self._writeln(" ...")
         self._writeln(
-            " {} different bytes found ({:.2%} different).".format(
-                self.diff_total, self.diff_ratio
-            )
+            f" {self.diff_total} different bytes found "
+            f"({self.diff_ratio:.2%} different)."
         )
 
 
@@ -1351,7 +1306,6 @@ class TableDataDiff(_BaseDiff):
 
             .. versionadded:: 2.0
         """
-
         self.ignore_fields = set(ignore_fields)
         self.numdiffs = numdiffs
         self.rtol = rtol
@@ -1581,18 +1535,15 @@ class TableDataDiff(_BaseDiff):
 
         if self.diff_values and self.numdiffs < self.diff_total:
             self._writeln(
-                " ...{} additional difference(s) found.".format(
-                    self.diff_total - self.numdiffs
-                )
+                f" ...{self.diff_total - self.numdiffs} additional difference(s) found."
             )
 
         if self.diff_total > self.numdiffs:
             self._writeln(" ...")
 
         self._writeln(
-            " {} different table data element(s) found ({:.2%} different).".format(
-                self.diff_total, self.diff_ratio
-            )
+            f" {self.diff_total} different table data element(s) found "
+            f"({self.diff_ratio:.2%} different)."
         )
 
 
@@ -1601,7 +1552,6 @@ def report_diff_keyword_attr(fileobj, attr, diffs, keyword, ind=0):
     Write a diff between two header keyword values or comments to the specified
     file-like object.
     """
-
     if keyword in diffs:
         vals = diffs[keyword]
         for idx, val in enumerate(vals):
@@ -1612,9 +1562,8 @@ def report_diff_keyword_attr(fileobj, attr, diffs, keyword, ind=0):
             else:
                 dup = f"[{idx + 1}]"
             fileobj.write(
-                fixed_width_indent(
-                    f" Keyword {keyword:8}{dup} has different {attr}:\n",
-                    ind,
+                textwrap.indent(
+                    f" Keyword {keyword:8}{dup} has different {attr}:\n", ind * "  "
                 )
             )
             report_diff_values(val[0], val[1], fileobj=fileobj, indent_width=ind + 1)
