@@ -32,11 +32,10 @@ from importlib.metadata import entry_points
 import numpy as np
 
 from astropy.units import Quantity
-from astropy.utils.decorators import deprecated
 from astropy.utils.exceptions import AstropyUserWarning
 
 from .optimizers import DEFAULT_ACC, DEFAULT_EPS, DEFAULT_MAXITER, SLSQP, Simplex
-from .spline import (  # noqa: F401
+from .spline import (
     SplineExactKnotsFitter,
     SplineInterpolateFitter,
     SplineSmoothingFitter,
@@ -58,6 +57,10 @@ __all__ = [
     "Fitter",
     "ModelLinearityError",
     "ModelsError",
+    "SplineExactKnotsFitter",
+    "SplineInterpolateFitter",
+    "SplineSmoothingFitter",
+    "SplineSplrepFitter",
 ]
 
 
@@ -70,7 +73,7 @@ OPTIMIZERS = [Simplex, SLSQP]
 
 class NonFiniteValueError(RuntimeError):
     """
-    Error raised when attempting to a non-finite value
+    Error raised when attempting to a non-finite value.
     """
 
 
@@ -105,8 +108,9 @@ class Covariance:
         if len(params) != 2:
             raise ValueError("Covariance must be indexed by two values.")
         if all(isinstance(item, str) for item in params):
-            i1, i2 = self.param_names.index(params[0]), self.param_names.index(
-                params[1]
+            i1, i2 = (
+                self.param_names.index(params[0]),
+                self.param_names.index(params[1]),
             )
         elif all(isinstance(item, int) for item in params):
             i1, i2 = params
@@ -160,7 +164,7 @@ class StandardDeviations:
 
 
 class ModelsError(Exception):
-    """Base class for model exceptions"""
+    """Base class for model exceptions."""
 
 
 class ModelLinearityError(ModelsError):
@@ -319,14 +323,14 @@ class Fitter(metaclass=_FitterMeta):
             raise ValueError("Expected an optimizer.")
         if statistic is None:
             raise ValueError("Expected a statistic function.")
-        if inspect.isclass(optimizer):
+        if isinstance(optimizer, type):
             # a callable class
             self._opt_method = optimizer()
         elif inspect.isfunction(optimizer):
             self._opt_method = optimizer
         else:
             raise ValueError("Expected optimizer to be a callable class or a function.")
-        if inspect.isclass(statistic):
+        if isinstance(statistic, type):
             self._stat_method = statistic()
         else:
             self._stat_method = statistic
@@ -372,7 +376,6 @@ class Fitter(metaclass=_FitterMeta):
         of a model.
         Fitter subclasses should implement this method.
         """
-
         raise NotImplementedError("Subclasses should implement this method.")
 
 
@@ -385,6 +388,7 @@ class LinearLSQFitter(metaclass=_FitterMeta):
     Uses `numpy.linalg.lstsq` to do the fitting.
     Given a model and data, fits the model to the data and changes the
     model's parameters. Keeps a dictionary of auxiliary fitting information.
+
     Notes
     -----
     Note that currently LinearLSQFitter does not support compound models.
@@ -504,7 +508,6 @@ class LinearLSQFitter(metaclass=_FitterMeta):
         Maps domain into window for a polynomial model which has these
         attributes.
         """
-
         if y is None:
             if hasattr(model, "domain") and model.domain is None:
                 model.domain = [x.min(), x.max()]
@@ -564,7 +567,6 @@ class LinearLSQFitter(metaclass=_FitterMeta):
             a copy of the input model with parameters set by the fitter
 
         """
-
         if not model.fittable:
             raise ValueError("Model must be a subclass of FittableModel")
 
@@ -590,14 +592,14 @@ class LinearLSQFitter(metaclass=_FitterMeta):
             x, y, z, n_models=len(model_copy), model_set_axis=model_copy.model_set_axis
         )
 
-        has_fixed = any(model_copy.fixed.values())
+        n_fixed = sum(model_copy.fixed.values())
 
         # This is also done by _convert_inputs, but we need it here to allow
         # checking the array dimensionality before that gets called:
         if weights is not None:
             weights = np.asarray(weights, dtype=float)
 
-        if has_fixed:
+        if n_fixed:
             # The list of fixed params is the complement of those being fitted:
             fixparam_indices = [
                 idx
@@ -632,7 +634,7 @@ class LinearLSQFitter(metaclass=_FitterMeta):
             # map domain into window
             if hasattr(model_copy, "domain"):
                 x = self._map_domain_window(model_copy, x)
-            if has_fixed:
+            if n_fixed:
                 lhs = np.asarray(
                     self._deriv_with_constraints(model_copy, fitparam_indices, x=x)
                 )
@@ -662,7 +664,7 @@ class LinearLSQFitter(metaclass=_FitterMeta):
             if hasattr(model_copy, "x_domain"):
                 x, y = self._map_domain_window(model_copy, x, y)
 
-            if has_fixed:
+            if n_fixed:
                 lhs = np.asarray(
                     self._deriv_with_constraints(model_copy, fitparam_indices, x=x, y=y)
                 )
@@ -724,7 +726,7 @@ class LinearLSQFitter(metaclass=_FitterMeta):
 
         # Subtract any terms fixed by the user from (a copy of) the RHS, in
         # order to fit the remaining terms correctly:
-        if has_fixed:
+        if n_fixed:
             if model_copy.col_fit_deriv:
                 fixderivs = np.asarray(fixderivs).T  # as for lhs above
             rhs = rhs - fixderivs.dot(fixparams)  # evaluate user-fixed terms
@@ -835,8 +837,7 @@ class LinearLSQFitter(metaclass=_FitterMeta):
         if (
             hasattr(model_copy, "_order")
             and len(model_copy) == 1
-            and not has_fixed
-            and rank != model_copy._order
+            and rank < (model_copy._order - n_fixed)
         ):
             warnings.warn("The fit may be poorly conditioned\n", AstropyUserWarning)
 
@@ -925,6 +926,7 @@ class FittingWithOutlierRemoval:
             Weights to be passed to the fitter.
         kwargs : dict, optional
             Keyword arguments to be passed to the fitter.
+
         Returns
         -------
         fitted_model : `~astropy.modeling.FittableModel`
@@ -934,7 +936,6 @@ class FittingWithOutlierRemoval:
             fitting iteration (False) and which were found to be outliers or
             were masked in the input (True).
         """
-
         # For single models, the data get filtered here at each iteration and
         # then passed to the fitter, which is the historical behavior and
         # works even for fitters that don't understand masked arrays. For model
@@ -1091,7 +1092,7 @@ class FittingWithOutlierRemoval:
 
 class _NonLinearLSQFitter(metaclass=_FitterMeta):
     """
-    Base class for Non-Linear least-squares fitters
+    Base class for Non-Linear least-squares fitters.
 
     Parameters
     ----------
@@ -1127,7 +1128,6 @@ class _NonLinearLSQFitter(metaclass=_FitterMeta):
             [model, [weights], [input coordinates]]
 
         """
-
         model = args[0]
         weights = args[1]
         fitter_to_model_params(model, fps, self._use_min_max_bounds)
@@ -1154,7 +1154,6 @@ class _NonLinearLSQFitter(metaclass=_FitterMeta):
         Set ``cov_matrix`` and ``stds`` attributes on model with parameter
         covariance matrix returned by ``optimize.leastsq``.
         """
-
         free_param_names = [
             x
             for x in model.fixed
@@ -1174,7 +1173,6 @@ class _NonLinearLSQFitter(metaclass=_FitterMeta):
         constraints, instead of using p directly, we set the parameter list in
         this function.
         """
-
         if weights is None:
             weights = 1.0
 
@@ -1242,15 +1240,26 @@ class _NonLinearLSQFitter(metaclass=_FitterMeta):
                     for _ in weights * np.array(model.fit_deriv(x, y, *params))
                 ]
 
-    def _compute_param_cov(self, model, y, init_values, cov_x, fitparams, farg):
+    def _compute_param_cov(
+        self, model, y, init_values, cov_x, fitparams, farg, weights=None
+    ):
         # now try to compute the true covariance matrix
         if (len(y) > len(init_values)) and cov_x is not None:
-            sum_sqrs = np.sum(self.objective_function(fitparams, *farg) ** 2)
-            dof = len(y) - len(init_values)
-            self.fit_info["param_cov"] = cov_x * sum_sqrs / dof
+            self.fit_info["param_cov"] = cov_x
+            if weights is None:
+                # if there are no measurement uncertainties given in `weights`,
+                # fall back on the default behavior in scipy.optimize.curve_fit
+                # when `absolute_sigma == False`. If there are uncertainties,
+                # assume they are "absolute" and not "relative".
+                # For details, see curve_fit:
+                #   https://github.com/scipy/scipy/blob/
+                #   c1ed5ece8ffbf05356a22a8106affcd11bd3aee0/scipy/
+                #   optimize/_minpack_py.py#L591-L602
+                sum_sqrs = np.sum(self.objective_function(fitparams, *farg) ** 2)
+                dof = len(y) - len(init_values)
+                self.fit_info["param_cov"] *= sum_sqrs / dof
         else:
             self.fit_info["param_cov"] = None
-
         if self._calc_uncertainties is True:
             if self.fit_info["param_cov"] is not None:
                 self._add_fitting_uncertainties(model, self.fit_info["param_cov"])
@@ -1258,7 +1267,7 @@ class _NonLinearLSQFitter(metaclass=_FitterMeta):
     def _run_fitter(self, model, farg, maxiter, acc, epsilon, estimate_jacobian):
         return None, None, None
 
-    def _filter_non_finite(self, x, y, z=None):
+    def _filter_non_finite(self, x, y, z=None, weights=None):
         """
         Filter out non-finite values in x, y, z.
 
@@ -1267,19 +1276,20 @@ class _NonLinearLSQFitter(metaclass=_FitterMeta):
         x, y, z : ndarrays
             x, y, and z with non-finite values filtered out.
         """
-
         MESSAGE = "Non-Finite input data has been removed by the fitter."
 
-        if z is None:
-            mask = np.isfinite(y)
-            if not np.all(mask):
-                warnings.warn(MESSAGE, AstropyUserWarning)
-            return x[mask], y[mask], None
-        else:
-            mask = np.isfinite(z)
-            if not np.all(mask):
-                warnings.warn(MESSAGE, AstropyUserWarning)
-            return x[mask], y[mask], z[mask]
+        mask = np.ones_like(x, dtype=bool) if weights is None else np.isfinite(weights)
+        mask &= np.isfinite(y) if z is None else np.isfinite(z)
+
+        if not np.all(mask):
+            warnings.warn(MESSAGE, AstropyUserWarning)
+
+        return (
+            x[mask],
+            y[mask],
+            None if z is None else z[mask],
+            None if weights is None else weights[mask],
+        )
 
     @fitter_unit_support
     def __call__(
@@ -1309,9 +1319,14 @@ class _NonLinearLSQFitter(metaclass=_FitterMeta):
         z : array, optional
            input coordinates
         weights : array, optional
-            Weights for fitting.
-            For data with Gaussian uncertainties, the weights should be
-            1/sigma.
+            Weights for fitting. For data with Gaussian uncertainties, the weights
+            should be 1/sigma.
+
+            .. versionchanged:: 5.3
+                Calculate parameter covariances while accounting for ``weights``
+                as "absolute" inverse uncertainties. To recover the old behavior,
+                choose ``weights=None``.
+
         maxiter : int
             maximum number of iterations
         acc : float
@@ -1338,12 +1353,11 @@ class _NonLinearLSQFitter(metaclass=_FitterMeta):
             a copy of the input model with parameters set by the fitter
 
         """
-
         model_copy = _validate_model(model, self.supported_constraints)
         model_copy.sync_constraints = False
 
         if filter_non_finite:
-            x, y, z = self._filter_non_finite(x, y, z)
+            x, y, z, weights = self._filter_non_finite(x, y, z, weights)
         farg = (
             model_copy,
             weights,
@@ -1353,7 +1367,9 @@ class _NonLinearLSQFitter(metaclass=_FitterMeta):
             model_copy, farg, maxiter, acc, epsilon, estimate_jacobian
         )
 
-        self._compute_param_cov(model_copy, y, init_values, cov_x, fitparams, farg)
+        self._compute_param_cov(
+            model_copy, y, init_values, cov_x, fitparams, farg, weights
+        )
 
         model.sync_constraints = True
         return model_copy
@@ -1673,7 +1689,6 @@ class SLSQPLSQFitter(Fitter):
             a copy of the input model with parameters set by the fitter
 
         """
-
         model_copy = _validate_model(model, self._opt_method.supported_constraints)
         model_copy.sync_constraints = False
         farg = _convert_input(x, y, z)
@@ -1743,7 +1758,6 @@ class SimplexLSQFitter(Fitter):
             a copy of the input model with parameters set by the fitter
 
         """
-
         model_copy = _validate_model(model, self._opt_method.supported_constraints)
         model_copy.sync_constraints = False
         farg = _convert_input(x, y, z)
@@ -1818,7 +1832,6 @@ class JointFitter(metaclass=_FitterMeta):
             args is always passed as a tuple from optimize.leastsq
 
         """
-
         lstsqargs = list(args)
         fitted = []
         fitparams = list(fps)
@@ -1874,7 +1887,6 @@ class JointFitter(metaclass=_FitterMeta):
         Fit data to these models keeping some of the parameters common to the
         two models.
         """
-
         from scipy import optimize
 
         if len(args) != reduce(lambda x, y: x + 1 + y + 1, self.modeldims):
@@ -1919,7 +1931,6 @@ class JointFitter(metaclass=_FitterMeta):
 
 def _convert_input(x, y, z=None, n_models=1, model_set_axis=0):
     """Convert inputs to float arrays."""
-
     x = np.asanyarray(x, dtype=float)
     y = np.asanyarray(y, dtype=float)
 
@@ -1989,7 +2000,6 @@ def fitter_to_model_params(model, fps, use_min_max_bounds=True):
         parameter with bounds.
         Default: True
     """
-
     _, fit_param_indices, _ = model_to_fit_params(model)
 
     has_tied = any(model.tied.values())
@@ -2047,11 +2057,6 @@ def fitter_to_model_params(model, fps, use_min_max_bounds=True):
                 model._array_to_parameters()
 
 
-@deprecated("5.1", "private method: _fitter_to_model_params has been made public now")
-def _fitter_to_model_params(model, fps):
-    return fitter_to_model_params(model, fps)
-
-
 def model_to_fit_params(model):
     """
     Convert a model instance's parameter array to an array that can be used
@@ -2061,7 +2066,6 @@ def model_to_fit_params(model):
     These may be a subset of the model parameters, if some of them are held
     constant or tied.
     """
-
     fitparam_indices = list(range(len(model.param_names)))
     model_params = model.parameters
     model_bounds = list(model.bounds.values())
@@ -2092,14 +2096,8 @@ def model_to_fit_params(model):
     return model_params, fitparam_indices, model_bounds
 
 
-@deprecated("5.1", "private method: _model_to_fit_params has been made public now")
-def _model_to_fit_params(model):
-    return model_to_fit_params(model)
-
-
 def _validate_constraints(supported_constraints, model):
     """Make sure model constraints are supported by the current fitter."""
-
     message = "Optimizer cannot handle {0} constraints."
 
     if any(model.fixed.values()) and "fixed" not in supported_constraints:
@@ -2125,7 +2123,6 @@ def _validate_model(model, supported_constraints):
     """
     Check that model and fitter are compatible and return a copy of the model.
     """
-
     if not model.fittable:
         raise ValueError("Model does not appear to be fittable.")
     if model.linear:
@@ -2159,7 +2156,6 @@ def populate_entry_points(entry_points):
     An explanation of entry points can be found `here
     <http://setuptools.readthedocs.io/en/latest/setuptools.html#dynamic-discovery-of-services-and-plugins>`_
     """
-
     for entry_point in entry_points:
         name = entry_point.name
         try:
@@ -2172,7 +2168,7 @@ def populate_entry_points(entry_points):
                 )
             )
         else:
-            if not inspect.isclass(entry_point):
+            if not isinstance(entry_point, type):
                 warnings.warn(
                     AstropyUserWarning(
                         f"Modeling entry point {name} expected to be a Class."
@@ -2182,7 +2178,7 @@ def populate_entry_points(entry_points):
                 if issubclass(entry_point, Fitter):
                     name = entry_point.__name__
                     globals()[name] = entry_point
-                    __all__.append(name)
+                    __all__.append(name)  # noqa: PYI056
                 else:
                     warnings.warn(
                         AstropyUserWarning(
@@ -2192,13 +2188,4 @@ def populate_entry_points(entry_points):
                     )
 
 
-def _populate_ep():
-    # TODO: Exclusively use select when Python minversion is 3.10
-    ep = entry_points()
-    if hasattr(ep, "select"):
-        populate_entry_points(ep.select(group="astropy.modeling"))
-    else:
-        populate_entry_points(ep.get("astropy.modeling", []))
-
-
-_populate_ep()
+populate_entry_points(entry_points().select(group="astropy.modeling"))
