@@ -2,6 +2,7 @@
 """
 Module to test fitting routines
 """
+
 # pylint: disable=invalid-name
 import os.path
 import unittest.mock as mk
@@ -160,7 +161,6 @@ class TestICheb2D:
 
 @pytest.mark.skipif(not HAS_SCIPY, reason="requires scipy")
 class TestJointFitter:
-
     """
     Tests the joint fitting routine using 2 gaussian models
     """
@@ -1203,9 +1203,9 @@ def test_optimizers(fitter_class):
             )
         assert fit_info == fitter._opt_method.fit_info
         if isinstance(fitter, SLSQPLSQFitter):
-            fitter._opt_method.acc == 1e-16
+            assert fitter._opt_method.acc == 1e-16
         else:
-            fitter._opt_method.acc == xtol
+            assert fitter._opt_method.acc == xtol
 
 
 @mk.patch.multiple(Optimization, __abstractmethods__=set())
