@@ -9,7 +9,13 @@ import pytest
 import yaml
 
 from astropy.utils import introspection
-from astropy.utils.introspection import find_current_module, find_mod_objs, minversion
+from astropy.utils.exceptions import AstropyDeprecationWarning
+from astropy.utils.introspection import (
+    find_current_module,
+    find_mod_objs,
+    isinstancemethod,
+    minversion,
+)
 
 
 def test_pkg_finder():
@@ -43,7 +49,12 @@ def test_find_current_mod():
 
 
 def test_find_mod_objs():
-    lnms, fqns, objs = find_mod_objs("astropy")
+    deprecation_message = (
+        "^The find_mod_objs function is deprecated and may be removed in "
+        r"a future version\.$"
+    )
+    with pytest.warns(AstropyDeprecationWarning, match=deprecation_message):
+        lnms, fqns, objs = find_mod_objs("astropy")
 
     # this import  is after the above call intentionally to make sure
     # find_mod_objs properly imports astropy on its own
@@ -54,27 +65,29 @@ def test_find_mod_objs():
     assert "test" in lnms
     assert astropy.test in objs
 
-    lnms, fqns, objs = find_mod_objs(__name__, onlylocals=False)
+    with pytest.warns(AstropyDeprecationWarning, match=deprecation_message):
+        lnms, fqns, objs = find_mod_objs(__name__, onlylocals=False)
     assert "namedtuple" in lnms
     assert "collections.namedtuple" in fqns
     assert namedtuple in objs
 
-    lnms, fqns, objs = find_mod_objs(__name__, onlylocals=True)
+    with pytest.warns(AstropyDeprecationWarning, match=deprecation_message):
+        lnms, fqns, objs = find_mod_objs(__name__, onlylocals=True)
     assert "namedtuple" not in lnms
     assert "collections.namedtuple" not in fqns
     assert namedtuple not in objs
 
 
 def test_minversion():
-    import numpy
+    import numpy as np
 
     good_versions = ["1.16", "1.16.1", "1.16.0.dev", "1.16dev"]
     bad_versions = ["100000", "100000.2rc1"]
     for version in good_versions:
-        assert minversion(numpy, version)
+        assert minversion(np, version)
         assert minversion("numpy", version)
     for version in bad_versions:
-        assert not minversion(numpy, version)
+        assert not minversion(np, version)
         assert not minversion("numpy", version)
 
     assert minversion(yaml, "3.1")
@@ -95,3 +108,34 @@ def test_find_current_module_bundle():
         assert find_current_module(0).__name__ == mod1
         assert find_current_module(1).__name__ == mod2
         assert find_current_module(0, True).__name__ == mod3
+
+
+def test_deprecated_isinstancemethod():
+    class MetaClass(type):
+        def a_classmethod(cls):
+            pass
+
+    class MyClass(metaclass=MetaClass):
+        def an_instancemethod(self):
+            pass
+
+        @classmethod
+        def another_classmethod(cls):
+            pass
+
+        @staticmethod
+        def a_staticmethod():
+            pass
+
+    deprecation_message = (
+        "^The isinstancemethod function is deprecated and may be removed in "
+        r"a future version\.$"
+    )
+    with pytest.warns(AstropyDeprecationWarning, match=deprecation_message):
+        assert isinstancemethod(MyClass, MyClass.a_classmethod) is False
+    with pytest.warns(AstropyDeprecationWarning, match=deprecation_message):
+        assert isinstancemethod(MyClass, MyClass.another_classmethod) is False
+    with pytest.warns(AstropyDeprecationWarning, match=deprecation_message):
+        assert isinstancemethod(MyClass, MyClass.a_staticmethod) is False
+    with pytest.warns(AstropyDeprecationWarning, match=deprecation_message):
+        assert isinstancemethod(MyClass, MyClass.an_instancemethod) is True
