@@ -2,7 +2,7 @@
 import warnings
 from contextlib import nullcontext
 
-import matplotlib
+import matplotlib as mpl
 import matplotlib.pyplot as plt
 import numpy as np
 import pytest
@@ -24,15 +24,14 @@ from astropy.visualization.wcsaxes.utils import get_coord_meta
 from astropy.wcs import WCS
 from astropy.wcs.wcsapi import HighLevelWCSWrapper, SlicedLowLevelWCS
 
-ft_version = Version(matplotlib.ft2font.__freetype_version__)
+ft_version = Version(mpl.ft2font.__freetype_version__)
 FREETYPE_261 = ft_version == Version("2.6.1")
 
 # We cannot use matplotlib.checkdep_usetex() anymore, see
 # https://github.com/matplotlib/matplotlib/issues/23244
 TEX_UNAVAILABLE = True
 
-# matplotlib 3.7 is not released yet.
-MATPLOTLIB_LT_3_7 = Version(matplotlib.__version__) < Version("3.6.99")
+MATPLOTLIB_LT_3_7 = Version(mpl.__version__) < Version("3.7")
 
 
 def teardown_function(function):
@@ -94,7 +93,7 @@ def test_no_numpy_warnings(ignore_matplotlibrc, tmp_path, grid_type):
     # (since this is normal).
     # BUT our own catch_warning was ignoring some warnings before, so now we
     # have to catch it. Otherwise, the pytest filterwarnings=error
-    # setting in setup.cfg will fail this test.
+    # setting in pyproject.toml will fail this test.
     # There are actually multiple warnings but they are all similar.
     with warnings.catch_warnings():
         warnings.filterwarnings(
@@ -412,11 +411,11 @@ def test_invalid_slices_errors(ignore_matplotlibrc):
 EXPECTED_REPR_1 = """
 <CoordinatesMap with 3 world coordinates:
 
-  index            aliases                type   unit wrap format_unit visible
-  ----- ------------------------------ --------- ---- ---- ----------- -------
-      0                   distmod dist    scalar      None                  no
-      1 pos.galactic.lon glon-car glon longitude  deg  360         deg     yes
-      2 pos.galactic.lat glat-car glat  latitude  deg None         deg     yes
+  index            aliases                type   ...    wrap   format_unit visible
+  ----- ------------------------------ --------- ... --------- ----------- -------
+      0                   distmod dist    scalar ...      None                  no
+      1 pos.galactic.lon glon-car glon longitude ... 360.0 deg         deg     yes
+      2 pos.galactic.lat glat-car glat  latitude ...      None         deg     yes
 
 >
  """.strip()
@@ -424,11 +423,11 @@ EXPECTED_REPR_1 = """
 EXPECTED_REPR_2 = """
 <CoordinatesMap with 3 world coordinates:
 
-  index            aliases                type   unit wrap format_unit visible
-  ----- ------------------------------ --------- ---- ---- ----------- -------
-      0                   distmod dist    scalar      None                 yes
-      1 pos.galactic.lon glon-car glon longitude  deg  360         deg     yes
-      2 pos.galactic.lat glat-car glat  latitude  deg None         deg     yes
+  index            aliases                type   ...    wrap   format_unit visible
+  ----- ------------------------------ --------- ... --------- ----------- -------
+      0                   distmod dist    scalar ...      None                 yes
+      1 pos.galactic.lon glon-car glon longitude ... 360.0 deg         deg     yes
+      2 pos.galactic.lat glat-car glat  latitude ...      None         deg     yes
 
 >
  """.strip()
@@ -601,3 +600,41 @@ def test_get_coord_range_nan_regression():
             [(-131.3193386797236, 180.0), (-44.02289164685554, 44.80732766607591)]
         ),
     )
+
+
+def test_imshow_error():
+    fig = plt.figure()
+    ax = fig.add_subplot(1, 1, 1, projection=WCS())
+    with pytest.raises(ValueError, match="Cannot use images with origin='upper"):
+        ax.imshow(np.ones(100).reshape(10, 10), origin="upper")
+
+
+def test_label_setting():
+    fig = plt.figure()
+    ax = fig.add_subplot(1, 1, 1, projection=WCS())
+    # Check both xlabel and label kwargs work
+    ax.set_xlabel(xlabel="label")
+    ax.set_xlabel(label="label")
+    # Check no label errors:
+    with pytest.raises(
+        TypeError, match=r"set_xlabel\(\) missing 1 required positional argument"
+    ):
+        ax.set_xlabel()
+
+    # Check both xlabel and label kwargs work
+    ax.set_ylabel(ylabel="label")
+    ax.set_ylabel(label="label")
+    # Check no label errors:
+    with pytest.raises(
+        TypeError, match=r"set_ylabel\(\) missing 1 required positional argument"
+    ):
+        ax.set_ylabel()
+
+
+def test_invisible_bbox():
+    fig = plt.figure()
+    ax = fig.add_subplot(1, 1, 1, projection=WCS())
+
+    assert ax.get_tightbbox(fig.canvas.get_renderer()) is not None
+    ax.set_visible(False)
+    assert ax.get_tightbbox(fig.canvas.get_renderer()) is None
